globals [clock]

to setup
  ca
  set clock 0
  ask patches
    [ set pcolor ((random colors) * 10) + 5 
        if (pcolor = 75)  ;; 75 is too close to another color so change it to 125
          [ set pcolor 125 ] ]
  do-plots
end

to go
  set (pcolor-of random-one-of patches) (pcolor-of random-one-of patches)
  set clock clock + 1
  do-plots
end

to do-plots
  if (clock mod 100 = 0) [ 
    set-current-plot "Patch Colors" 
    set-current-plot-pen "color5" 
    plot count patches with [pcolor = 5]
    set-current-plot-pen "color15" 
    plot count patches with [pcolor = 15]
    set-current-plot-pen "color25" 
    plot count patches with [pcolor = 25]
    set-current-plot-pen "color35" 
    plot count patches with [pcolor = 35]
    set-current-plot-pen "color45" 
    plot count patches with [pcolor = 45]
    set-current-plot-pen "color55" 
    plot count patches with [pcolor = 55]
    set-current-plot-pen "color65" 
    plot count patches with [pcolor = 65]
    set-current-plot-pen "color125" 
    plot count patches with [pcolor = 125]
    set-current-plot-pen "color85" 
    plot count patches with [pcolor = 85]
    set-current-plot-pen "color95" 
    plot count patches with [pcolor = 95]
  ]
end 


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2001 by Uri Wilensky.  Updated 2002.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2001).  NetLogo GenDrift P global model.
; http://ccl.northwestern.edu/netlogo/models/GenDriftPglobal.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
333
19
605
291
8
8
16.0
1
10
0
0

CC-WINDOW
13
138
319
289
Command Center

BUTTON
98
23
153
56
go
go
T
1
T
OBSERVER

BUTTON
37
23
92
56
setup
setup
NIL
1
T
OBSERVER

SLIDER
42
67
151
100
colors
colors
2
10
5
1
1
NIL

PLOT
32
306
580
523
Patch Colors
Time
Number
0.0
100.0
0.0
75.0
true
false
PENS
"color5" 1.0 0 -7566196 true
"color15" 1.0 0 -65536 true
"color25" 1.0 0 -44544 true
"color35" 1.0 0 -6524078 true
"color45" 1.0 0 -256 true
"color55" 1.0 0 -11352576 true
"color65" 1.0 0 -16711936 true
"color125" 1.0 0 -65281 true
"color85" 1.0 0 -16711681 true
"color95" 1.0 0 -16745473 true

MONITOR
193
37
274
86
Clock / 100
clock / 100
0
1

@#$#@#$#@
WHAT IS IT? 
----------- 
This model is an example of random selection.  It shows that patches that randomly exchange colors converge on a single color. The idea, explained in more detail in Dennett's "Darwin's Dangerous Idea", is that trait drifts can occur without any particular purpose or 'selecting pressure'.

The model starts with a random distribution of colors.  It then randomly picks a patch to copy a color from, and randomly picks a patch to change to the copied color.  The coordinates are global, so it could happen that the same patch is selected for both roles.  

After enough time passes, a color will gain a slight dominance. By statistical advantage, a dominate color becomes more likely to win the entire grid.  However, because the process is random, there will usually be many series of dominant colors before one color finally wins.  A key aspect is that once a color disappears -- becomes extinct -- it can never reappear.  

HOW TO USE IT: 
-------------- 
The "setup" button intializes the model. 
The "go" button starts it, and runs continuously. 
Use the "colors" slider to select the number of competing colors.

THINGS TO NOTICE: 
---------------- 
Notice that often colors can become quite dominant but still fail to win the grid. 

EXTENDING THE MODEL: 
-------------------- 
Explore other rules by which patches might randomly influence each other.  What does it take for one color to "win out"?    

RELATED MODELS: 
--------------- 
GenDrift (P local)
GenDrift (T interact)
GenDrift (T reproduce)

CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2001).  NetLogo GenDrift P global model. http://ccl.northwestern.edu/netlogo/models/GenDriftPglobal. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

turtle
true
5
Polygon -11352576 true true 151 122 225 46 255 76 180 151 256 226 226 256 151 181 76 256 46 227 122 152 46 76 76 47
Circle -11352576 true true 68 68 166
Polygon -11352576 true true 151 1 109 81 192 81

@#$#@#$#@
NetLogo 1.2pre2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
